searchData={"items":[{"type":"module","title":"Sooth.Predictor","doc":"Provides the Predictor module for Sooth.\n\nThis is the main entry point for the Sooth library.","ref":"Sooth.Predictor.html"},{"type":"function","title":"Sooth.Predictor.count/2","doc":"Return the number of times the context has been observed.","ref":"Sooth.Predictor.html#count/2"},{"type":"function","title":"Parameters - Sooth.Predictor.count/2","doc":"- `predictor` - The predictor that will count the context.\n- `id` - A number that provides a context for observations.","ref":"Sooth.Predictor.html#count/2-parameters"},{"type":"function","title":"Examples - Sooth.Predictor.count/2","doc":"iex> predictor =\n    ...>  Sooth.Predictor.new(0)\n    ...>  |> Sooth.Predictor.observe(2, 3)\n    ...>  |> Sooth.Predictor.observe(2, 3)\n    ...>  |> Sooth.Predictor.observe(2, 2)\n    iex> Sooth.Predictor.count(predictor, 2)\n    3\n    iex> Sooth.Predictor.count(predictor, 3)\n    0","ref":"Sooth.Predictor.html#count/2-examples"},{"type":"function","title":"Sooth.Predictor.distribution/2","doc":"Return a stream that yields each observed event within the context together with its\nprobability.","ref":"Sooth.Predictor.html#distribution/2"},{"type":"function","title":"Parameters - Sooth.Predictor.distribution/2","doc":"- `predictor` - The predictor that will calculate the distribution.\n- `id` - A number that provides a context for the distribution.","ref":"Sooth.Predictor.html#distribution/2-parameters"},{"type":"function","title":"Examples - Sooth.Predictor.distribution/2","doc":"iex> predictor = Sooth.Predictor.new(0)\n    ...> |> Sooth.Predictor.observe(0, 3)\n    ...> |> Sooth.Predictor.observe(0, 2)\n    iex> Sooth.Predictor.distribution(predictor, 0)\n    [{2, 0.5}, {3, 0.5}]\n    iex> Sooth.Predictor.distribution(predictor, 1)\n    nil","ref":"Sooth.Predictor.html#distribution/2-examples"},{"type":"function","title":"Sooth.Predictor.fetch_random_select/2","doc":"Return a random event that may occur in the given context. The event is selected by\nrandomly choosing a number between 1 and the `count` of the context.\n\nReturns:\n  An event that has been previously observed in the given context, or the error_event\n  if the `count` of the context is zero.","ref":"Sooth.Predictor.html#fetch_random_select/2"},{"type":"function","title":"Parameters - Sooth.Predictor.fetch_random_select/2","doc":"- `predictor` - The predictor that will select the event.\n- `id` - A number that provides a context for observations.","ref":"Sooth.Predictor.html#fetch_random_select/2-parameters"},{"type":"function","title":"Sooth.Predictor.frequency/3","doc":"Return a number indicating the frequency that the event has been observed within the\ngiven context.","ref":"Sooth.Predictor.html#frequency/3"},{"type":"function","title":"Parameters - Sooth.Predictor.frequency/3","doc":"- `predictor` - The predictor that will calculate the frequency.\n- `id` - A number that provides a context for the frequency.\n- `event` - A number representing the observed event.","ref":"Sooth.Predictor.html#frequency/3-parameters"},{"type":"function","title":"Examples - Sooth.Predictor.frequency/3","doc":"iex> predictor = Sooth.Predictor.new(0)\n    ...> |> Sooth.Predictor.observe(0, 3)\n    ...> |> Sooth.Predictor.observe(0, 3)\n    ...> |> Sooth.Predictor.observe(0, 4)\n    ...> |> Sooth.Predictor.observe(0, 5)\n    ...> |> Sooth.Predictor.observe(1, 2)\n    iex> Sooth.Predictor.frequency(predictor, 0, 3)\n    0.5\n    iex> Sooth.Predictor.frequency(predictor, 0, 4)\n    0.25\n    iex> Sooth.Predictor.frequency(predictor, 0, 5)\n    0.25\n    iex> Sooth.Predictor.frequency(predictor, 1, 2)\n    1.0","ref":"Sooth.Predictor.html#frequency/3-examples"},{"type":"function","title":"Sooth.Predictor.new/1","doc":"Returns a new Sooth.Predictor.","ref":"Sooth.Predictor.html#new/1"},{"type":"function","title":"Parameters - Sooth.Predictor.new/1","doc":"- `error_event` - The event to be returned by #select when no observations have been made for the context.","ref":"Sooth.Predictor.html#new/1-parameters"},{"type":"function","title":"Examples - Sooth.Predictor.new/1","doc":"iex> Sooth.Predictor.new(2)\n    #Sooth.Predictor","ref":"Sooth.Predictor.html#new/1-examples"},{"type":"function","title":"Sooth.Predictor.observe/3","doc":"Register an observation of the given event within the given context.","ref":"Sooth.Predictor.html#observe/3"},{"type":"function","title":"Parameters - Sooth.Predictor.observe/3","doc":"- `predictor` - The predictor that will observe the event.\n- `id` - A number that provides a context for the event, allowing the predictor to maintain observation statistics for different contexts.\n- `event` - A number representing the observed event.","ref":"Sooth.Predictor.html#observe/3-parameters"},{"type":"function","title":"Options - Sooth.Predictor.observe/3","doc":"- `:include_count` - If this option is given, the function will return the count of the context.\n- `:include_statistic` - If this option is given, the function will return the statistic of the event.","ref":"Sooth.Predictor.html#observe/3-options"},{"type":"function","title":"Examples - Sooth.Predictor.observe/3","doc":"iex> predictor = Sooth.Predictor.new(0)\n    iex> {predictor, count} = Sooth.Predictor.observe(predictor, 0, 3, :include_count)\n    iex> predictor\n    #Sooth.Predictor }>\n    iex> count\n    1","ref":"Sooth.Predictor.html#observe/3-examples"},{"type":"function","title":"Sooth.Predictor.observe/4","doc":"","ref":"Sooth.Predictor.html#observe/4"},{"type":"function","title":"Sooth.Predictor.select/3","doc":"Return an event that may occur in the given context, based on the limit, which should be\nbetween 1 and #count. The event is selected by iterating through all observed events for\nthe context, subtracting the observation count of each event from the limit until it is\nzero or less.\n\nReturns:\n  An event that has been previously observed in the given context, or the error_event\n  if the `count` of the context is zero, or if limit exceeds the `count` of the context.","ref":"Sooth.Predictor.html#select/3"},{"type":"function","title":"Parameters - Sooth.Predictor.select/3","doc":"- `predictor` - The predictor that will select the event.\n- `id` - A number that provides a context for observations.\n- `limit` - The total number of event observations to be analysed before returning a event.","ref":"Sooth.Predictor.html#select/3-parameters"},{"type":"function","title":"Examples - Sooth.Predictor.select/3","doc":"iex> predictor = Sooth.Predictor.new(99)\n    ...> |> Sooth.Predictor.observe(1, 4)\n    ...> |> Sooth.Predictor.observe(1, 3)\n    ...> |> Sooth.Predictor.observe(1, 4)\n    ...> |> Sooth.Predictor.observe(1, 5)\n    iex> Sooth.Predictor.select(predictor, 1, 1)\n    3\n    iex> Sooth.Predictor.select(predictor, 1, 2)\n    4\n    iex> Sooth.Predictor.select(predictor, 1, 3)\n    4\n    iex> Sooth.Predictor.select(predictor, 1, 4)\n    5\n    iex> Sooth.Predictor.select(predictor, 0, 0)\n    99\n    iex> Sooth.Predictor.select(predictor, 1, 5)\n    99","ref":"Sooth.Predictor.html#select/3-examples"},{"type":"function","title":"Sooth.Predictor.size/2","doc":"Return the number of different events that have been observed within the given context.","ref":"Sooth.Predictor.html#size/2"},{"type":"function","title":"Parameters - Sooth.Predictor.size/2","doc":"- `predictor` - The predictor that will calculate the size.\n- `id` - A number that provides a context for observations.","ref":"Sooth.Predictor.html#size/2-parameters"},{"type":"function","title":"Examples - Sooth.Predictor.size/2","doc":"iex> predictor = Sooth.Predictor.new(0)\n    ...>  |> Sooth.Predictor.observe(0, 3)\n    ...>  |> Sooth.Predictor.observe(0, 3)\n    ...>  |> Sooth.Predictor.observe(0, 3)\n    ...>  |> Sooth.Predictor.observe(0, 1)\n    iex> Sooth.Predictor.size(predictor, 0)\n    2","ref":"Sooth.Predictor.html#size/2-examples"},{"type":"function","title":"Sooth.Predictor.surprise/3","doc":"Return a number indicating the surprise received by the predictor when it observed\nthe given event within the given context. Note that nil will be returned if the\nevent has never been observed within the context.\n\nReturns:\n  The surprise, which is calculated to be the Shannon pointwise mutual information\n  of the event according to the `distribution` over the context.","ref":"Sooth.Predictor.html#surprise/3"},{"type":"function","title":"Parameters - Sooth.Predictor.surprise/3","doc":"- `predictor` - The predictor that will calculate the surprise.\n- `id` - A number that provides a context for observations.\n- `event` - A number representing the observed event.","ref":"Sooth.Predictor.html#surprise/3-parameters"},{"type":"function","title":"Examples - Sooth.Predictor.surprise/3","doc":"iex> predictor = Sooth.Predictor.new(9)\n    ...> |> Sooth.Predictor.observe(0, 3)\n    ...> |> Sooth.Predictor.observe(0, 3)\n    ...> |> Sooth.Predictor.observe(0, 5)\n    ...> |> Sooth.Predictor.observe(1, 2)\n    ...> |> Sooth.Predictor.observe(1, 3)\n    ...> |> Sooth.Predictor.observe(1, 4)\n    iex> Sooth.Predictor.surprise(predictor, 0, 3)\n    {:ok, 0.5849625007211563}\n    iex> Sooth.Predictor.surprise(predictor, 0, 4)\n    :error\n    iex> Sooth.Predictor.surprise(predictor, 1, 2)\n    {:ok, 1.5849625007211563}","ref":"Sooth.Predictor.html#surprise/3-examples"},{"type":"function","title":"Sooth.Predictor.uncertainty/2","doc":"Return a number indicating how uncertain the predictor is about which event is likely\nto be observed after the given context. Note that nil will be returned if the context\nhas never been observed.\n\nReturns:\n  The uncertainty, which is calculated to be the Shannon entropy of the `distribution`\n  over the context.","ref":"Sooth.Predictor.html#uncertainty/2"},{"type":"function","title":"Parameters - Sooth.Predictor.uncertainty/2","doc":"- `predictor` - The predictor that will calculate the uncertainty.\n- `id` - A number that provides a context for observations.","ref":"Sooth.Predictor.html#uncertainty/2-parameters"},{"type":"function","title":"Examples - Sooth.Predictor.uncertainty/2","doc":"iex> predictor = Sooth.Predictor.new(0)\n    ...> |> Sooth.Predictor.observe(0, 3)\n    ...> |> Sooth.Predictor.observe(0, 3)\n    ...> |> Sooth.Predictor.observe(0, 4)\n    ...> |> Sooth.Predictor.observe(0, 5)\n    ...> |> Sooth.Predictor.observe(1, 2)\n    ...> |> Sooth.Predictor.observe(1, 3)\n    ...> |> Sooth.Predictor.observe(1, 4)\n    iex> Sooth.Predictor.uncertainty(predictor, 0)\n    {:ok, 1.5}\n    iex> Sooth.Predictor.uncertainty(predictor, 1)\n    {:ok, 1.584962500721156}\n    iex> Sooth.Predictor.uncertainty(predictor, 2)\n    :error","ref":"Sooth.Predictor.html#uncertainty/2-examples"},{"type":"type","title":"Sooth.Predictor.t/0","doc":"A Predictor of error_event/contexts","ref":"Sooth.Predictor.html#t:t/0"},{"type":"module","title":"Sooth.Context","doc":"Provides the Context module for Sooth.\n\nThis module is mainly used for internal implementation details of the Sooth.Predictor module\nand should not be used directly.","ref":"Sooth.Context.html"},{"type":"function","title":"Sooth.Context.fetch_statistic/2","doc":"Fetch a statistic in a context.\n\nThis is an implementation detail and should not be used directly.","ref":"Sooth.Context.html#fetch_statistic/2"},{"type":"function","title":"Examples - Sooth.Context.fetch_statistic/2","doc":"iex> context = Sooth.Context.new(0)\n    iex> Sooth.Context.fetch_statistic(context, 3)\n    :error\n    iex> {context, _} = Sooth.Context.observe(context, 3)\n    iex> Sooth.Context.fetch_statistic(context, 3)\n    {:ok, %Sooth.Statistic{count: 1, event: 3}}","ref":"Sooth.Context.html#fetch_statistic/2-examples"},{"type":"function","title":"Sooth.Context.find_statistic/2","doc":"Find a statistic in a context.\n\nThis is an implementation detail and should not be used directly.","ref":"Sooth.Context.html#find_statistic/2"},{"type":"function","title":"Sooth.Context.new/1","doc":"Create a new context.","ref":"Sooth.Context.html#new/1"},{"type":"function","title":"Examples - Sooth.Context.new/1","doc":"iex> Sooth.Context.new(0)\n    #Sooth.Context","ref":"Sooth.Context.html#new/1-examples"},{"type":"function","title":"Sooth.Context.observe/2","doc":"Observe an event in a context.","ref":"Sooth.Context.html#observe/2"},{"type":"function","title":"Examples - Sooth.Context.observe/2","doc":"iex> context = Sooth.Context.new(0)\n    iex> {context, _} = Sooth.Context.observe(context, 3)\n    iex> context\n    #Sooth.Context","ref":"Sooth.Context.html#observe/2-examples"},{"type":"function","title":"Sooth.Context.to_list/1","doc":"Convert a context to a list of statistics.","ref":"Sooth.Context.html#to_list/1"},{"type":"function","title":"Examples - Sooth.Context.to_list/1","doc":"iex> context = Sooth.Context.new(0)\n    iex> Sooth.Context.to_list(context)\n    []\n    iex> {context, _} = Sooth.Context.observe(context, 3)\n    iex> Sooth.Context.to_list(context)\n    [%Sooth.Statistic{count: 1, event: 3}]","ref":"Sooth.Context.html#to_list/1-examples"},{"type":"type","title":"Sooth.Context.t/0","doc":"A Context of id/count/statistics","ref":"Sooth.Context.html#t:t/0"},{"type":"module","title":"Sooth.Statistic","doc":"A simple data structure for tracking event counts.","ref":"Sooth.Statistic.html"},{"type":"function","title":"Sooth.Statistic.increment/1","doc":"Increment the count of a statistic.","ref":"Sooth.Statistic.html#increment/1"},{"type":"function","title":"Examples - Sooth.Statistic.increment/1","doc":"iex> Sooth.Statistic.increment(%Sooth.Statistic{event: 0, count: 0})\n    %Sooth.Statistic{event: 0, count: 1}","ref":"Sooth.Statistic.html#increment/1-examples"},{"type":"function","title":"Sooth.Statistic.new/2","doc":"Create a new statistic.","ref":"Sooth.Statistic.html#new/2"},{"type":"function","title":"Examples - Sooth.Statistic.new/2","doc":"iex> Sooth.Statistic.new(0, 0)\n    %Sooth.Statistic{event: 0, count: 0}","ref":"Sooth.Statistic.html#new/2-examples"},{"type":"type","title":"Sooth.Statistic.t/0","doc":"A Statistic of event/count","ref":"Sooth.Statistic.html#t:t/0"},{"type":"extras","title":"Introduction","doc":"# Introduction\n\nSooth is a minimal stochastic predictive model library for Elixir. It provides a simple yet powerful way to make predictions based on observed events in different contexts.\n\nThis library has been ported from the [Ruby/C Sooth library](https://github.com/kranzky/sooth) with optimizations for Elixir and a focus on performance to handle large MegaHal trainings and models.","ref":"introduction.html"},{"type":"extras","title":"Key Features - Introduction","doc":"- Simple API for observing events and making predictions\n- Efficient storage and retrieval of event statistics\n- Calculation of event frequencies, surprises, and uncertainties","ref":"introduction.html#key-features"},{"type":"extras","title":"Setup - Introduction","doc":"To start using Sooth in your project, add it to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:sooth, \"~> 0.5.0\"}\n  ]\nend\n```","ref":"introduction.html#setup"},{"type":"extras","title":"Basic Usage - Introduction","doc":"In Megahal (a [Markov model](https://en.wikipedia.org/wiki/Markov_model) based chatbot), the `Sooth.Predictor` is used to train the model using `Sooth.Predictor.observe/3` on sentence structures in pairs of words to build up statistics of what word comes next. The `Sooth.Predictor.fetch_random_select/2` is then used to assemble them. Sentences are broken down into word pairs, assigned integers mapping them to a dictionary, and observed:\n\n```elixir\nimport Sooth.Predictor\n\nmodel = Sooth.Predictor.new(0)\n\n# Each integer maps to a normalized word in a dictionary, 1 representing the sentence start/finish\n# This is what 'learning' a sentence in pairs of words looks like.\nmodel = \n  model\n  |> Sooth.Predictor.observe([1, 10], 2)\n  |> Sooth.Predictor.observe([10, 2], 3)\n  |> Sooth.Predictor.observe([2, 3], 5)\n  |> Sooth.Predictor.observe([3, 5], 9)\n  |> Sooth.Predictor.observe([5, 9], 1)\n\n# Assuming many more sentences were broken down and trained, there'd be multiple possible\n# next events given an id to randomly select from.\n\nnext_word_token = Sooth.Predictor.fetch_random_select(model, [10, 2])\n\n```","ref":"introduction.html#basic-usage"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}